#ifndef EXPRESSION_H
#define EXPRESSION_H

#include <string>
#include <cassert>
#include <sstream>
#include "Type.h"
#include "TypeError.h"
#include "ExpressionVisitor.h"

// The expressions in our mini-language.
class Expression {
public:
    virtual Type accept(ExpressionVisitor<Type, TypeError>& v) = 0;
    virtual std::string toString() const = 0;
};

// Here follow all the possible expressions:

class TrueExpression : public Expression {
public:
    std::string toString() const override;
    Type accept(ExpressionVisitor<Type, TypeError>& v) override;
};

class FalseExpression : public Expression {
public:
    std::string toString() const override;
    Type accept(ExpressionVisitor<Type, TypeError>& v) override;
};

class IntExpression : public Expression {
public:
    IntExpression(int val);
    std::string toString() const override;
    Type accept(ExpressionVisitor<Type, TypeError>& v) override;

private:
    int value;
};

class NegExpression : public Expression {
public:
    NegExpression(Expression* expression);
    std::string toString() const override;
    Type accept(ExpressionVisitor<Type, TypeError>& v) override;

private:
    Expression* e;
};

class AddExpression : public Expression {
public:
    AddExpression(Expression* expr1, Expression* expr2);
    std::string toString() const override;
    Type accept(ExpressionVisitor<Type, TypeError>& v) override;

private:
    Expression* e1;
    Expression* e2;
};

class OrExpression : public Expression {
public:
    OrExpression(Expression* expr1, Expression* expr2);
    std::string toString() const override;
    Type accept(ExpressionVisitor<Type, TypeError>& v) override;

private:
    Expression* e1;
    Expression* e2;
};

class EqualsExpression : public Expression {
public:
    EqualsExpression(Expression* expr1, Expression* expr2);
    std::string toString() const override;
    Type accept(ExpressionVisitor<Type, TypeError>& v) override;

private:
    Expression* e1;
    Expression* e2;
};

class IfThenElseExpression : public Expression {
public:
    IfThenElseExpression(Expression* condition, Expression* thenExpr, Expression* elseExpr);
    std::string toString() const override;
    Type accept(ExpressionVisitor<Type, TypeError>& v) override;

private:
    Expression* cond;
    Expression* then;
    Expression* else_;
};

#endif // EXPRESSION_H
